@@ -34,6 +34,10 @@ MAX_RECURSION_DEPTH = 2
 # All the successful HTTP status codes from RFC 7231 & 4918
 HTTP_SUCCESS = (200, 201, 202, 203, 204, 205, 206, 207)
 
+# JSON Schema validator and format checker used for JSON Schema validation
+JSONSCHEMA_VALIDATOR = jsonschema.Draft4Validator
+FORMAT_CHECKER = jsonschema.draft4_format_checker
+
 
 class RestClient(object):
     """Unified OpenStack RestClient class
@@ -436,7 +440,10 @@ class RestClient(object):
                                    req_body, resp_body, caller_name, extra)
 
     def _parse_resp(self, body):
-        body = json.loads(body)
+        try:
+            body = json.loads(body)
+        except ValueError:
+            return body
 
         # We assume, that if the first value of the deserialized body's
         # item set is a dict or a list, that we just return the first value
@@ -666,12 +673,18 @@ class RestClient(object):
             raise exceptions.InvalidContentType(str(resp.status))
 
         if resp.status == 401:
+            if parse_resp:
+                resp_body = self._parse_resp(resp_body)
             raise exceptions.Unauthorized(resp_body)
 
         if resp.status == 403:
+            if parse_resp:
+                resp_body = self._parse_resp(resp_body)
             raise exceptions.Forbidden(resp_body)
 
         if resp.status == 404:
+            if parse_resp:
+                resp_body = self._parse_resp(resp_body)
             raise exceptions.NotFound(resp_body)
 
         if resp.status == 400:
@@ -731,7 +744,7 @@ class RestClient(object):
             if resp.status == 501:
                 raise exceptions.NotImplemented(message)
             else:
-                raise exceptions.ServerFault(message)
+                raise exceptions.ServerFault(resp_body, message=message)
 
         if resp.status >= 400:
             raise exceptions.UnexpectedResponseCode(str(resp.status))
@@ -796,7 +809,9 @@ class RestClient(object):
             body_schema = schema.get('response_body')
             if body_schema:
                 try:
-                    jsonschema.validate(body, body_schema)
+                    jsonschema.validate(body, body_schema,
+                                        cls=JSONSCHEMA_VALIDATOR,
+                                        format_checker=FORMAT_CHECKER)
                 except jsonschema.ValidationError as ex:
                     msg = ("HTTP response body is invalid (%s)") % ex
                     raise exceptions.InvalidHTTPResponseBody(msg)
@@ -809,7 +824,9 @@ class RestClient(object):
             header_schema = schema.get('response_header')
             if header_schema:
                 try:
-                    jsonschema.validate(resp, header_schema)
+                    jsonschema.validate(resp, header_schema,
+                                        cls=JSONSCHEMA_VALIDATOR,
+                                        format_checker=FORMAT_CHECKER)
                 except jsonschema.ValidationError as ex:
                     msg = ("HTTP response header is invalid (%s)") % ex
                     raise exceptions.InvalidHTTPResponseHeader(msg)
